/*
 * File:   MPU9250.c
 * Author: Sanja
 *
 * Created on December 10, 2022, 2:34 PM
 */


#include <xc.h>
#include "MPU9250.h"

uint8_t MPU9250_who_am_i(void)
{
    return I2C1_Read1ByteRegister(MPU9250_ADDR, WHO_AM_I);
}

void MPU9250_init(void)
{
    // power down mode off
    I2C1_Write1ByteRegister(MPU9250_ADDR, PWR_MGMT_1, 0x00);
    
    // Enable accel and gyro
    I2C1_Write1ByteRegister(MPU9250_ADDR, PWR_MGMT_2, 0x00);
    
    // Sampling rate div 1
    I2C1_Write1ByteRegister(MPU9250_ADDR, SMPLRT_DIV, 0x00);
    
    //8g accel range
    I2C1_Write1ByteRegister(MPU9250_ADDR, ACCEL_CONFIG, 0x10);
    
    //500dps gyro range
    I2C1_Write1ByteRegister(MPU9250_ADDR, GYRO_CONFIG, 0x08);
}

void MPU9250_read_accel(MPU9250_t *sensor)
{
    uint8_t tmp[6];
    
    I2C1_ReadDataBlock(MPU9250_ADDR, ACCEL_XOUT_H, tmp, 6);
   
    sensor->accel_x_raw = ((int16_t)tmp[0])<<8 | tmp[1];
    sensor->accel_y_raw = ((int16_t)tmp[2])<<8 | tmp[3];
    sensor->accel_z_raw = ((int16_t)tmp[4])<<8 | tmp[5];
    
    sensor->ax = ((float)sensor->accel_x_raw)/4096.0;
    sensor->ay = ((float)sensor->accel_y_raw)/4096.0;
    sensor->az = ((float)sensor->accel_z_raw)/4096.0;
    
}

void MPU9250_read_gyro(MPU9250_t *sensor)
{
    uint8_t tmp[6];
    
    I2C1_ReadDataBlock(MPU9250_ADDR, GYRO_XOUT_H, tmp, 6);
    
    sensor->gyro_x_raw = tmp[0]<<8 | tmp[1];
    sensor->gyro_y_raw = tmp[2]<<8 | tmp[3];
    sensor->gyro_z_raw = tmp[4]<<8 | tmp[5];
    
    sensor->ax = sensor->accel_x_raw/65.5;
    sensor->ay = sensor->accel_y_raw/65.5;
    sensor->az = sensor->accel_z_raw/65.5;
}
